import processing.core.*; 
import processing.xml.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class VJ_TOOL extends PApplet {

/*
* VJ TOOL
*
* This programm creates a permutation with signs and provides multiple functionalities to
* let the user interact with the permutation and provides sound input
*
* @name Interactive Permutation
* @author Sebastian Pape <sebastian.pape@zhdk.ch>
* @version 09.12.2010
*/

/*
 * VARIABLES */
PermutationManager       permutationManager;
                           
/*
* SETUP */
public void setup()
{
  // SIZE
  size(screen.width, screen.height);
  
  // SMOOTH
  smooth();
  
  // PERMUTATION MANAGER
  permutationManager = new PermutationManager();
}

/*
 * DRAW */
public void draw()
{
  // BACKGROUND
  background(255);
  
  // NO MOUSE CURSOR
  noCursor();
  
  // SET UP PERMUTATION
  permutationManager.setUpPermutation(); 
  
  // DRAW SIGNS
  permutationManager.drawSigns();
}


/*
* PERMUTATION MANAGER 
*
* This class manages the permutation
*
* @name Permutation Manager
* @author Sebastian Pape <sebastian.pape@zhdk.ch>
* @version 09.12.2010
*/

class PermutationManager {
  
  /*
  * VARIABLES */
  int        signDiameter = 130;
  ArrayList  signs = new ArrayList();

  /*
  * CONSTRUCTOR */
  PermutationManager() {
  }
  
  /*
  * SET UP PERMUTATION */
  public void setUpPermutation() {
    // CALCULATE AMOUNT OF NEEDED SIGNS
    int _signsHorizontal = screen.width / signDiameter;
    int _signsVertical = screen.height / signDiameter;
    
    // MARGIN TO FILL EMPTY SPACE
    int _signMarginHorizontal = (screen.width - (_signsHorizontal * signDiameter)) / (_signsHorizontal-1);
    int _signMarginVertical = (screen.height - (_signsVertical * signDiameter)) / (_signsVertical-1);
    
    // SET UP SIGNS PER LINES
    signs =  new ArrayList();
    for(int _v = 0; _v < _signsVertical; _v++) {
      for(int _h = 0; _h < _signsHorizontal; _h++) {
        
        // X
        int _x = signDiameter/2 + _h * (signDiameter + _signMarginHorizontal);
        
        // Y
        int _y = signDiameter/2 + _v * (signDiameter + _signMarginVertical);

        // SIGN
        Sign _sign = new Sign(_x, _y, signDiameter);
        signs.add(_sign);
        
      }
    }
  }
  
  /*
  * DRAW SIGNS */
  public void drawSigns() {
    for(int i = 0; i < signs.size(); i++) {  
      Sign _sign = (Sign)signs.get(i);
      _sign.draw();
    }
  }
  
}
/*
* SIGNS
*
* This class manages the sign itselfes and draw it with given parameters
*
* @name Sign
* @author Sebastian Pape <sebastian.pape@zhdk.ch>
* @version 09.12.2010
*/

class Sign {
  
  /*
  * VARIABLES */
  int    x;
  int    y;
  int    diameter;
  int    type = 1;
  
  /*
  * CONSTRUCTOR */
  Sign(int _x, int _y, int _diameter) {
    x = _x;
    y = _y;
    diameter = _diameter;
  }
  
  /*
  * GET MOUSE DEGREE */
  public float getMouseDegree(float _x, float _y) {
    float mouseDegree = degrees(atan2(mouseY - _y, mouseX - _x));
  
    if (mouseDegree < 0) {
      // negative
      if(mouseDegree >= -90) {
        mouseDegree = (mouseDegree+90);
      } 
      else {
        mouseDegree = (mouseDegree+450);
      }
    } 
    else {
      mouseDegree = (mouseDegree+90);
    }
  
    return mouseDegree;
  }
  
  /*
  * GET MOUSE DISTANCE */
  public float getMouseDistance(float _x, float _y) {
    float mouseDistance = dist(_x, _y, mouseX, mouseY);;
    return mouseDistance;
  }
  
  /*
  * DRAW */
  public void draw() {
    
    // TRANSLATE
    pushMatrix();
      translate(x,y);
      
      // ROTATE
      pushMatrix();
        rotate(radians(getMouseDegree(x,y)));
        
        // TRANSFORMATION
        float _transformation = norm(getMouseDistance(x,y),0, screen.width);
        
        // DRAW
        switch(type) {
          case 1:
            drawSignType1(_transformation);
          break;
          default: 
            drawSignType1(_transformation);
          break; 
        }
        
      popMatrix();// TRANSLATE
    popMatrix();// ROTATE
  }
  
  /* 
  * DRAW SIGN TYPE 1 */
  public void drawSignType1(float _transformation){
      /* CIRCLE 1 BLACK */
      stroke(0);
      fill(0);
      ellipse(
              0,
              0,
              diameter,
              diameter
      );
      
      /* CIRCLE 2 WHITE */
      stroke(0);
      fill(255);
      ellipse(
              0,
              -_transformation*diameter*.01f,
              diameter*.95f - diameter*_transformation*.05f,
              diameter*.95f - diameter*_transformation*.05f
      );
      
       /* CIRCLE 3 BLACK */
      stroke(0);
      fill(0);
      ellipse(
              0,
              -_transformation*diameter*.045f,
              diameter*.33f + diameter*_transformation*.53f,
              diameter*.33f + diameter*_transformation*.53f
       );
      
       /* CIRCLE 4 WHITE */
      stroke(0);
      fill(255);
      ellipse(
              0,
              -_transformation*diameter*.2f,
              diameter*.40f + diameter*_transformation*.1f,
              diameter*.40f + diameter*_transformation*.1f
      );
      
       /* CIRCLE 5 BLACK */
      stroke(0);
      fill(0);
      ellipse(
              0,
              -_transformation*diameter*.28f,
              diameter*.15f + diameter*_transformation*.2f,
              diameter*.15f + diameter*_transformation*.2f
      );
  }
  
}
  static public void main(String args[]) {
    PApplet.main(new String[] { "--present", "--bgcolor=#666666", "--hide-stop", "VJ_TOOL" });
  }
}
